# Generated by Django 3.2.13 on 2022-05-24 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("smpp_gateway", "0002_mtmessagestatus"),
    ]

    operations = [
        migrations.AlterField(
            model_name="momessage",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="params",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="params",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="delivery_report",
            field=models.BinaryField(
                help_text="The delivery_report we receive from the MC (via its deliver_sm).",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="message_id",
            field=models.CharField(
                blank=True,
                default="",
                help_text="The message_id we receive from the MC (via its submit_sm_resp).",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="sequence_number",
            field=models.IntegerField(
                help_text="The initial sequence_number established by us when the message was sent."
            ),
        ),
    ]
