# Generated by Django 3.2.16 on 2022-12-06 21:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("smpp_gateway", "0004_translations"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="mtmessagestatus",
            options={
                "verbose_name": "mobile-terminated message status",
                "verbose_name_plural": "mobile-terminated message statuses",
            },
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="command_status",
            field=models.IntegerField(
                choices=[
                    (0, "No Error"),
                    (1, "Message Length is invalid"),
                    (2, "Command Length is invalid"),
                    (3, "Invalid Command ID"),
                    (4, "Incorrect BIND Status for given command"),
                    (5, "ESME Already in Bound State"),
                    (6, "Invalid Priority Flag"),
                    (7, "<Desc Not Set>"),
                    (8, "System Error"),
                    (10, "Invalid Source Address"),
                    (11, "Invalid Destination Address"),
                    (12, "Invalid Message ID"),
                    (13, "Bind Failed"),
                    (14, "Invalid Password"),
                    (15, "Invalid System ID"),
                    (17, "Cancel SM Failed"),
                    (19, "Replace SM Failed"),
                    (20, "Message Queue is full"),
                    (21, "Invalid Service Type"),
                    (51, "Invalid number of destinations"),
                    (52, "Invalid Distribution List name"),
                    (64, "Invalid Destination Flag (submit_multi)"),
                    (
                        66,
                        "Invalid Submit With Replace request (replace_if_present_flag set)",
                    ),
                    (67, "Invalid esm_class field data"),
                    (68, "Cannot submit to Distribution List"),
                    (69, "submit_sm or submit_multi failed"),
                    (72, "Invalid Source address TON"),
                    (73, "Invalid Source address NPI"),
                    (80, "Invalid Destination address TON"),
                    (81, "Invalid Destination address NPI"),
                    (83, "Invalid system_type field"),
                    (84, "Invalid replace_if_present flag"),
                    (85, "Invalid number of messages"),
                    (88, "Throttling error (ESME has exceeded allowed message limits)"),
                    (97, "Invalid Scheduled Delivery Time"),
                    (98, "Invalid message validity period (Expiry Time)"),
                    (99, "Predefined Message is invalid or not found"),
                    (100, "ESME received Temporary App Error Code"),
                    (101, "ESME received Permanent App Error Code"),
                    (102, "ESME received Reject Message Error Code"),
                    (103, "query_sm request failed"),
                    (192, "Error in the optional part of the PDU body"),
                    (193, "Optional Parameter not allowed"),
                    (194, "Invalid Parameter Length"),
                    (195, "Expected Optional Parameter missing"),
                    (196, "Invalid Optional Parameter Value"),
                    (254, "Delivery Failure (used data_sm_resp)"),
                    (255, "Unknown Error"),
                ],
                null=True,
                verbose_name="command status",
            ),
        ),
    ]
