# Generated by Django 4.0.5 on 2022-06-30 11:40

import django.db.models.deletion

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rapidsms", "0004_auto_20150801_2138"),
        ("smpp_gateway", "0003_big_pks_and_help_texts"),
    ]

    operations = [
        migrations.AlterField(
            model_name="momessage",
            name="backend",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="rapidsms.backend",
                verbose_name="backend",
            ),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="create_time",
            field=models.DateTimeField(verbose_name="create time"),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="modify_time",
            field=models.DateTimeField(verbose_name="modify time"),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="params",
            field=models.JSONField(verbose_name="params"),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="short_message",
            field=models.BinaryField(verbose_name="short message"),
        ),
        migrations.AlterField(
            model_name="momessage",
            name="status",
            field=models.CharField(
                choices=[
                    ("new", "New"),
                    ("processing", "Processing"),
                    ("done", "Done"),
                ],
                max_length=32,
                verbose_name="status",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="backend",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="rapidsms.backend",
                verbose_name="backend",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="create_time",
            field=models.DateTimeField(verbose_name="create time"),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="modify_time",
            field=models.DateTimeField(verbose_name="modify time"),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="params",
            field=models.JSONField(verbose_name="params"),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="short_message",
            field=models.TextField(verbose_name="short message"),
        ),
        migrations.AlterField(
            model_name="mtmessage",
            name="status",
            field=models.CharField(
                choices=[
                    ("new", "New"),
                    ("sending", "Sending"),
                    ("sent", "Sent"),
                    ("delivered", "Delivered"),
                    ("error", "Error"),
                ],
                max_length=32,
                verbose_name="status",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="backend",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="rapidsms.backend",
                verbose_name="backend",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="command_status",
            field=models.IntegerField(null=True, verbose_name="command status"),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="create_time",
            field=models.DateTimeField(verbose_name="create time"),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="delivery_report",
            field=models.BinaryField(
                help_text="The delivery_report we receive from the MC (via its deliver_sm).",
                null=True,
                verbose_name="delivery report",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="message_id",
            field=models.CharField(
                blank=True,
                default="",
                help_text="The message_id we receive from the MC (via its submit_sm_resp).",
                max_length=255,
                verbose_name="message id",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="modify_time",
            field=models.DateTimeField(verbose_name="modify time"),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="mt_message",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="smpp_gateway.mtmessage",
                verbose_name="mt message",
            ),
        ),
        migrations.AlterField(
            model_name="mtmessagestatus",
            name="sequence_number",
            field=models.IntegerField(
                help_text="The initial sequence_number established by us when the message was sent.",
                verbose_name="sequence number",
            ),
        ),
    ]
